#include <iostream>
#include <fstream>
#include <sstream>
#include <string>
#include <iomanip>
#include <limits>
#include <math.h>
#include <TArc.h>
#include <TArrow.h>
#include <TBox.h>
#include <TBrowser.h>
#include <TCanvas.h>
#include <TCondition.h>
#include <TDatime.h>
#include <TDirectory.h>
#include <TEllipse.h>
#include <TGraphErrors.h>
#include <TGButton.h>
#include <TGButtonGroup.h>
#include <TGClient.h>
#include <TGComboBox.h>
#include <TGProgressBar.h>
#include <TGLabel.h>
#include <TGTextEntry.h>
#include <TGFrame.h>
#include <TGraph.h>
#include <TGraphErrors.h>
#include <TF1.h>
#include <TFile.h>
#include <TH1D.h>
#include <TH2D.h>
#include <TLatex.h>
#include <TLeaf.h>
#include <TLine.h>
#include <TObjectTable.h>
#include <TPaveText.h>
#include <TPolyLine.h>
#include <TProfile.h>
#include <TRandom2.h>
#include <TRootEmbeddedCanvas.h>
#include <TStyle.h>
#include "TSystem.h"
#include <TVectorD.h>
#include <TDatime.h>
#include <sys/types.h>
#include <time.h>
#include <TROOT.h>
#include <TError.h>
#include "TVirtualFFT.h"
#include <complex>

using namespace std;

/* folders with scope data files content ->
  2011.09.06 - preamp old and new, ISF and CSV test, 1000+1000 
  2011.09.09 - preamp old and new, full scan, ISF format, 1000+1000 
  2011.09.21 - miniFEB, isolated pulse
  2011.09.21a  miniFEB, two gains, 1000+1000
  2013-08-22miniFEB new miniFEB study with different attenuators and 100Ohm cable 
*/
bool   smoothAmp=true;
bool   smoothPha=false;
double longRange=9000.;
double pi2=4.0*asin(1.0);
int    iqq=2;

double freqCut0=0.01;  // MHZ
double freqCut1=75.;  // MHZ
double freqCut2=75.; // MHZ
double fftMX=0.;
int    jAtt3=1;  // where to pick the 'low att' FFT

std::ifstream indata;
std::ofstream outfile;

bool   loadFile(char *fileName, int iOff, double *data, double *time, bool getTime);
double getYbin(char *fileName);
double fitIN(double *par, double t);
double fitTail(double *par, double t);
void saveTransferFunction(char* dirTF, int iCh, double Att, 
    int nSizeLoad, double bin001, double *slope, double *shift);

void fftRestore(int Nsamp, double fBin, double fCut, 
                TVirtualFFT *FFT, TVirtualFFT *FFTback, 
                double *output, double *aI,    double *aR, double *aA, double *aP, 
                double *scale,  double *shift, double *input); 

void fftGetSpectrum(int Nsamp, double fBin, double fCut, double &fftMax, TVirtualFFT *FFT, 
                    double *signal, double *aI,    double *aR, double *aA, double *aP) {
    
  FFT->SetPoints(signal);
  FFT->Transform();
  FFT->GetPointsComplex(aR, aI);
  double wFFT=1./sqrt(double(Nsamp));
  for (int i=0; i<Nsamp; i++) {
    aR[i]*=wFFT;
    aI[i]*=wFFT;
    aA[i]=sqrt(aR[i]*aR[i]+aI[i]*aI[i]);
    aP[i]=atan2(aI[i], aR[i]);
    if (aA[i]>fftMax && i*fBin<fCut) fftMax=aA[i];
  }
}

void fftFilter10MHz(int Nsamp, double fBin, double *a) {
// filters out 10, 20, ... MHz spikes
double fMax=fBin*(Nsamp/2+1);
for (double fX=10.; fX<fMax; fX+=10.) {
  int iClose=int(fX/fBin+0.5);
  int i0=iClose-1;
  int i1=iClose+1;
  if (i0<0 || i1>Nsamp/2+1) continue;
//  printf(" f=%5.1f a[%d, %d, %d] = { %f\t, %f->%f\t, %f\t}\n", fClose, i0, iClose, i1, a[i0], a[iClose], 0.5*(a[i0]+a[i1]), a[i1]);
  a[iClose]=0.5*(a[i0]+a[i1]);
}
}
void fftSmoothA(int Nsamp, double fBin, double f0, double fSlope, double *a) {
// filters tochastic flukes in amplitude spectrum
for (int i=1; i<Nsamp/2; i++) {
  double fX=fBin*i;
  if (fX<f0) continue;
  double w=0.33+0.67*exp(-(fX-f0)/fSlope);
  a[i]=(w*a[i]+(1.0-w)*(a[i-1]+a[i+1]))/(2.0-w);
//  if (i%1000==0) {
//    printf("  smooth f=%6.1fMHz w=%5.2f -> %5.2f + %5.2f testSum=%6.3f\n",
//            fX, w, w/(2.0-w), (1.0-w)/(2.0-w), w/(2.0-w)+2.0*(1.0-w)/(2.0-w));
//  }
}
}

void fftSmoothP(int Nsamp, double fBin, double f0, double fSlope, double *a) {
// filters tochastic flukes in phase shift
for (int i=1; i<Nsamp/2; i++) {
  double fX=fBin*i;
  if (fX<f0) continue;
  double w=0.5+0.5*exp(-(fX-f0)/fSlope);
  double an=a[i+1];
  if (a[i]-an >  pi2/2.0) an+=pi2;
  if (a[i]-an < -pi2/2.0) an-=pi2; 
  an=(1-w)*a[i]+w*an;
  if (an> pi2/2.0) an-=pi2;
  if (an<-pi2/2.0) an+=pi2;
  a[i+1]=an;
//  if (i%1000==0) {
//    printf("  smooth f=%6.1fMHz w=%5.2f -> %5.2f + %5.2f testSum=%6.3f\n",
//            fX, w, w/(2.0-w), (1.0-w)/(2.0-w), w/(2.0-w)+2.0*(1.0-w)/(2.0-w));
//  }
}
}




void fftGetConversion(int Nsamp,     double fBin,  double fCut, double &slopeMax, 
                      double *Ain,   double *Aout, double *Pin, double *Pout, 
                      double *slope, double *shift) {
  for (int i=0; i<Nsamp; i++) {
    if (Ain[i]==0.0) {
      slope[i]=0.0;
    } else {
      slope[i]=Aout[i]/Ain[i];
    }
    if (slope[i]>slopeMax && i*fBin<fCut) slopeMax=slope[i];
    //printf("[fftGetConversion] %10d f=%d\t in=%f\t out=%f\t slope=%f\t max=%f\n", i, i*fBin, Ain[i], Aout[i], slope[i], slopeMax); 

    shift[i]=Pout[i]-Pin[i];
    while (shift[i]>pi2/2.)   shift[i]-=pi2;
    while (shift[i]<=-pi2/2.) shift[i]+=pi2;
  }
/*
  for (int i=1; i<Nsamp-1; i++) { // remove phase shift jutter
    double dd=0.5*(shift[i-1]+shift[i+1]) - shift[i];
    if (dd>0.75*pi2)       shift[i]+=pi2;
    else if (dd<-0.75*pi2) shift[i]-=pi2;
  }
*/
}
//==============================================================
void MyExecuteEvent(Int_t event, Int_t px, Int_t py, TObject *sel);

//==============================================================
void simulation1ns024cabledFFT() {
//==============================================================
bool V_mA=true;
double bin001    = 0.01; // dT=1ns, N=100000; T=100us
V_mA=false;

char dirTF[200];
sprintf(dirTF, "/home/savin/HiLum/analysis2012/FFT/TransitionFunction/bench2013/");

TCanvas *inputCanvas             = new TCanvas("Input",       "Input pulses",         0,   0, 800, 1130);

TCanvas *displayCanvas           = new TCanvas("Display",     "Display",            200,   0, 800, 1130);
TCanvas *displayCanvasZoom       = new TCanvas("DisplayZoom", "DisplayZoom",        400, 800, 800, 1130);

TCanvas *displayCanvasSignal     = new TCanvas("DisplaySignal",     "Display",     600,   0, 800, 1130);
TCanvas *displayCanvasSignalZoom = new TCanvas("DisplaySignalZoom", "DisplayZoom", 800, 800, 800, 1130);

//TCanvas *spectraCanvasInput      = new TCanvas("spectraCanvasIn",    "Input spectra",  600,   0, 800, 1130);      
TCanvas *reconstructionFullBand  = new TCanvas("reconstructionFullBand",  "Reconstruction, Full Band", 1000,   0, 800, 1130);  
TCanvas *reconstruction40MHz     = new TCanvas("reconstruction40",  "Reconstruction, 40MHz", 2000,   0, 800, 1130);
TCanvas *simulationCanvas        = new TCanvas("simulationCanvas",   "1:300 sum simulation", 2800,   0, 800, 1130);
TCanvas *simulationCanvas1       = new TCanvas("simulationCanvas1",  "1:300 sum sim events", 2400,   0, 800, 1130);        
TCanvas *simulationCanvas2       = new TCanvas("simulationCanvas2",  "1:300 sum sim events", 2000,   0, 800, 1130);        




inputCanvas             -> Connect("ProcessedEvent(Int_t,Int_t,Int_t,TObject*)", 0, 0, 
                                   "MyExecuteEvent(Int_t,Int_t,Int_t,TObject*)");
displayCanvas           -> Connect("ProcessedEvent(Int_t,Int_t,Int_t,TObject*)", 0, 0, 
                                   "MyExecuteEvent(Int_t,Int_t,Int_t,TObject*)");
displayCanvasZoom       -> Connect("ProcessedEvent(Int_t,Int_t,Int_t,TObject*)", 0, 0, 
                                   "MyExecuteEvent(Int_t,Int_t,Int_t,TObject*)");
displayCanvasSignal     -> Connect("ProcessedEvent(Int_t,Int_t,Int_t,TObject*)", 0, 0, 
                                   "MyExecuteEvent(Int_t,Int_t,Int_t,TObject*)");
displayCanvasSignalZoom -> Connect("ProcessedEvent(Int_t,Int_t,Int_t,TObject*)", 0, 0, 
                                   "MyExecuteEvent(Int_t,Int_t,Int_t,TObject*)");
simulationCanvas      -> Connect("ProcessedEvent(Int_t,Int_t,Int_t,TObject*)", 0, 0, 
                                 "MyExecuteEvent(Int_t,Int_t,Int_t,TObject*)");
simulationCanvas1     -> Connect("ProcessedEvent(Int_t,Int_t,Int_t,TObject*)", 0, 0, 
                                 "MyExecuteEvent(Int_t,Int_t,Int_t,TObject*)");
//spectraCanvasInput      -> Connect("ProcessedEvent(Int_t,Int_t,Int_t,TObject*)", 0, 0, 
//                                   "MyExecuteEvent(Int_t,Int_t,Int_t,TObject*)");
reconstructionFullBand     -> Connect("ProcessedEvent(Int_t,Int_t,Int_t,TObject*)", 0, 0, 
                                   "MyExecuteEvent(Int_t,Int_t,Int_t,TObject*)");
reconstruction40MHz     -> Connect("ProcessedEvent(Int_t,Int_t,Int_t,TObject*)", 0, 0, 
                                   "MyExecuteEvent(Int_t,Int_t,Int_t,TObject*)");

TH1F *randomSpectrum = new TH1F("randomSpectrum", "Channel Contribution", 100, 0., 1.0);
randomSpectrum->GetXaxis()->SetTitle("Channel weight");
randomSpectrum->GetYaxis()->SetTitle("Events");
randomSpectrum->SetLineWidth(2);

TH1F *outputPeak     = new TH1F("outputPeak",     "Output Sum", 100, 0., 4.0);
outputPeak->GetXaxis()->SetTitle("Response Peak, V");
outputPeak->GetYaxis()->SetTitle("Events");
outputPeak->SetLineWidth(2);

TH1F *recoPeak       = new TH1F("recoPeak",       "Input Current", 100, 0., 30.0);
recoPeak->GetXaxis()->SetTitle("Reconstructed Peak, mA");
recoPeak->GetYaxis()->SetTitle("Events");
recoPeak->SetLineWidth(2);

TH1F *recoQ       = new TH1F("recoQ",       "Charge in 50ns", 120, 0., 1100.0);
recoQ->GetXaxis()->SetTitle("Collected charge, pC");
recoQ->GetYaxis()->SetTitle("Events");





TLine  *dummyLine = new TLine();
TLatex *dummyLatex= new TLatex();

int nAttn=6; // number of attenuators
int nPt[6];
double dTin[6], delay[6], ff[6];
double inputMin[6]; // Min of the input pulse, mA
double inputMax[6]; // Max of the input pulse, mA
double inputCurrent[6]={0.05, 0.1, 0.3, 1.0, 5., 5.}; // input current peak, mA

double fileAtt[1000]={1000*-1.0};    // -1 - for unused indexes
double Attenuator[6]={1., 3., 10., 30., 100., 300.};
int    iFilePulsed[1000]={1000*-100};  // -1 for all pulsed
int    iFileCabled[1000];  // -1 for unused cable
int    iMapScope[1000][4];
double gainCorr[1000][4]; // gain correction of pulsed channel(s) in the run
double s0[5]={5*0.}, s1[5]={5*0.}, s2[5]={5*0.}; // average gains for 4 channels

int iChScope; // scope input matching the miniFEB input we are looking at
double yBin[4];   // channel
double aMax[4];
double aMin[4];
double ped0[4];
double cMax[4];
double cMin[4];
//double pedC[4];
double cableDelay=200.; //ns
double aMinG, aMaxG;
double cMinG, cMaxG;
double dd;

double *input;
double *input1;
double *input2;
double *data;
double *data1;
double *data0;
double *data2;
double *data3;
double *dataZ;
double *dataSum;  // sum of signals from 'all pulsed'
double *dataSumS;  // sum of signals from 'all pulsed', simulated
double *dataChnS;  // channel, simulated
double *dataSumX; // crosstalk sum from 'other pulsed'
double *dataX;
double *reco;  // reconstructed with current Sum TF
double *reco3; // reconstructed with 'low att' Sum TF
double *reco_40;  // reconstructed with current Sum TF,   40MHz frequency cut
double *reco3_40; // reconstructed with 'low att' Sum TF, 40MHz frequency cut
double *recoMIN;  // signals, corresponding to largest ans smallest reconstructed signals
double *recoMAX;
double *outMIN;
double *outMAX;
double weightMAX[4]={4*0.0}, weightMIN[4]={4*0.0};
double *time;

int iOff, iOff1;
int nSizeLoadIn=10000;
int nSizeLoad=100000;
int nChan=5;
double tDisp0=-25.;
double tDisp1=800.;
double tDisp0z=-5.;
double tDisp1z=100.;
double inputTrange=700.;
double t0in=1040.;

char title[200];
char fileName[200];

input    = (double*) calloc(nSizeLoadIn*nAttn,sizeof(double));
data     = (double*) calloc(nSizeLoad*nChan, sizeof(double));
data1    = (double*) calloc(nSizeLoad,       sizeof(double));
data2    = (double*) calloc(nSizeLoad,       sizeof(double));
data3    = (double*) calloc(nSizeLoad*nChan, sizeof(double)); 
dataX    = (double*) calloc(nSizeLoad*nChan, sizeof(double));
data0    = (double*) calloc(nSizeLoad*nChan, sizeof(double));   // channel is pulsed alone
dataZ    = (double*) calloc(nSizeLoad*nChan*nChan, sizeof(double)); // X-talk, received by ch(i) from other channels 
dataChnS = (double*) calloc(nSizeLoad*nChan, sizeof(double));
dataSumX = (double*) calloc(nSizeLoad,       sizeof(double));
dataSum  = (double*) calloc(nSizeLoad,       sizeof(double));
dataSumS = (double*) calloc(nSizeLoad,       sizeof(double));
time     = (double*) calloc(nSizeLoad,       sizeof(double));
input1   = (double*) calloc(nSizeLoad,       sizeof(double));
input2   = (double*) calloc(nSizeLoad,       sizeof(double));
reco     = (double*) calloc(nSizeLoad,       sizeof(double));
reco3    = (double*) calloc(nSizeLoad,       sizeof(double));
reco_40  = (double*) calloc(nSizeLoad,       sizeof(double));
reco3_40 = (double*) calloc(nSizeLoad,       sizeof(double));
recoMIN  = (double*) calloc(nSizeLoad,       sizeof(double));
recoMAX  = (double*) calloc(nSizeLoad,       sizeof(double));
outMIN   = (double*) calloc(nSizeLoad,       sizeof(double));
outMAX   = (double*) calloc(nSizeLoad,       sizeof(double));

double *fftRin; // input 100k, workspace for full signal  
double *fftIin;
double *fftAin;
double *fftPin;
double *fftRout; //
double *fftIout;
double *fftAout;
double *fftPout;
double *fftRrec; //
double *fftIrec;
double *fftArec;
double *fftPrec;


fftRin  = (double*) calloc(nSizeLoad,       sizeof(double));
fftIin  = (double*) calloc(nSizeLoad,       sizeof(double));
fftAin  = (double*) calloc(nSizeLoad,       sizeof(double));
fftPin  = (double*) calloc(nSizeLoad,       sizeof(double));
fftRout = (double*) calloc(nSizeLoad,       sizeof(double));
fftIout = (double*) calloc(nSizeLoad,       sizeof(double));
fftAout = (double*) calloc(nSizeLoad,       sizeof(double));
fftPout = (double*) calloc(nSizeLoad,       sizeof(double));
fftRrec = (double*) calloc(nSizeLoad,       sizeof(double));
fftIrec = (double*) calloc(nSizeLoad,       sizeof(double));
fftArec = (double*) calloc(nSizeLoad,       sizeof(double));
fftPrec = (double*) calloc(nSizeLoad,       sizeof(double));

double *slope;
double *shift;
double *slopeX; // transfer function for X-talk
double *shiftX;
double *slope3; // storage for 'low att' TF
double *shift3;
double ws;
slope  =  (double*) calloc(nSizeLoad,       sizeof(double));
shift  =  (double*) calloc(nSizeLoad,       sizeof(double));
slope3 =  (double*) calloc(nSizeLoad,       sizeof(double));
shift3 =  (double*) calloc(nSizeLoad,       sizeof(double));
slopeX =  (double*) calloc(nSizeLoad,       sizeof(double));
shiftX =  (double*) calloc(nSizeLoad,       sizeof(double));




TVirtualFFT *FFT1;
TVirtualFFT *FFT_BACK1;
// for 1ns sampling
FFT1      = TVirtualFFT::FFT(1, &nSizeLoad, "R2C ES K");
FFT_BACK1 = TVirtualFFT::FFT(1, &nSizeLoad, "C2R M K");



TH1D *dummy = new TH1D("dummy", "dummy", nSizeLoad+1, 0., 100.);
dummy->GetXaxis()->SetTitle("Time, ns");
dummy->SetStats(0);
dummy->SetLineStyle(2);
TH1D *dummyF = new TH1D("dummyF", "dummyF", nSizeLoad+1, freqCut0, freqCut1);
dummyF->GetXaxis()->SetTitle("Frequency, MHz");
dummyF->SetStats(0);
dummyF->SetLineStyle(2);


for (int i=0; i<1000; i++) {
  for (int j=0; j<4; j++) {
     gainCorr[i][j]=0.;
   }
}

for (int i=0; i<nSizeLoadIn*nAttn; i++) input[i]=0.0;

for (int iAtt=0; iAtt<nAttn; iAtt++) {

  //sprintf(fileName, "2013.09.01/input%dmV.dat", int(1000*inputCurrent[iAtt]));
  sprintf(fileName, "2013.09.01/input%.0fmV.dat", 1000.*inputCurrent[iAtt]);
  indata.open(fileName, ios::in);
  if (!indata.is_open()) {
    printf("%s not found, bye\n", fileName);
    exit(-1);
  }

  indata >> nPt[iAtt] >> dTin[iAtt] >> delay[iAtt] >> ff[iAtt];
  dTin[iAtt]*=1.0e9;
  iOff=nSizeLoadIn*iAtt;
  inputMin[iAtt]=1.0e10;
  inputMax[iAtt]=-1.0e10;
  double ped=0.;
  double w=0.;
  for (int i=0; i<nPt[iAtt]; i++) {
    indata >> input[i+iOff]; // Volts;
    input[i+iOff]/=1.05;     //since the injector resistor is 1.05kOhm, mA;
    if (i*dTin[iAtt]-t0in<-50.) {
      w+=1.0;
      ped+=input[i+iOff];
    }
  }
  ped/=w;
  for (int i=0; i<nPt[iAtt]; i++) {
    input[i+iOff]-=ped;
    if (inputMin[iAtt]>input[i+iOff]) inputMin[iAtt]=input[i+iOff];
    if (inputMax[iAtt]<input[i+iOff]) inputMax[iAtt]=input[i+iOff];
  }
  indata.close();
  indata.clear(); 
}
for (int iAtt=0; iAtt<6; iAtt++) {inputCurrent[iAtt]/=1.05*Attenuator[iAtt];} // 1kOhm+50Ohm in series

indata.open("2013.09.01.map", ios::in);
while (!indata.eof()) {
  int jFile, iAtt, iInput, iPulsed, in1, in2, in3, in4, iCab;
  indata >> jFile >> iAtt >> iInput >> iPulsed >> in1 >> in2 >> in3 >> in4 >> iCab; 
//  printf(" %5d  %5d  %5d  %5d  %5d  %5d  %5d  %5d  %5d \n",
//            jFile, iAtt, iInput, iPulsed, in1, in2, in3, in4, iCab);
  if (iAtt==-1) break;
  iFilePulsed[jFile]=iPulsed;
  fileAtt[jFile]=iAtt;
  iMapScope[jFile][0]=in1;
  iMapScope[jFile][1]=in2;
  iMapScope[jFile][2]=in3;
  iMapScope[jFile][3]=in4;
  iFileCabled[jFile]=iCab;
}
indata.close();
indata.clear(); 

sprintf(fileName, "gainCorrection2013.09.12.dat");
indata.open(fileName, ios::in);
if (!indata.is_open()) {
  printf("%s not open, bye\n", fileName);
  indata.clear();
  exit(-1);
}
for (int iFile=0; iFile<1000; iFile++) {
  indata >> iFile;
  if (indata.eof()) break;
  for (int iCh=0; iCh<4; iCh++) {
    indata >> gainCorr[iFile][iCh];
    if (gainCorr[iFile][iCh] != 0.0) {
      s0[iCh]+=1.0;
      s1[iCh]+=gainCorr[iFile][iCh];
      s2[iCh]+=gainCorr[iFile][iCh]*gainCorr[iFile][iCh];
    }
  }
}
indata.close();
indata.clear();
for (int iCh=0; iCh<4; iCh++) {
  s0[4]+=s0[iCh];
  s1[4]+=s1[iCh];
  s2[4]+=s2[iCh];
  s1[iCh]/=s0[iCh];
  s2[iCh]/=s0[iCh];
  s2[iCh]-=s1[iCh]*s1[iCh];
  s2[iCh]=sqrt(s2[iCh]);
  printf("Chan#%d gain=%5.2f(+/-%.4f) rms=%.4f\n", iCh, s1[iCh], s2[iCh]/sqrt(s0[iCh]-1.0), s2[iCh]); 
}
s1[4]/=s0[4];
s2[4]/=s0[4];
s2[4]-=s1[4]*s1[4];
s2[4]=sqrt(s2[4]);
printf("====== gain=%5.2f(+/-%.4f) rms=%.4f\n", s1[4], s2[4]/sqrt(s0[4]-1.0), s2[4]); 

for (int iFile=0; iFile<1000; iFile++) {
  double w=0.;
  for (int iCh=0; iCh<4; iCh++) {
    if (gainCorr[iFile][iCh] != 0.0) {
      gainCorr[iFile][iCh]=s1[4]/gainCorr[iFile][iCh];
      w+=gainCorr[iFile][iCh];
    }
  }
  if (w != 0.0) {
    printf(" file#%03d 1:%.0f", iFile, fileAtt[iFile]);
    for (int iCh=0; iCh<4; iCh++) {
      if (gainCorr[iFile][iCh] != 0.) printf(" Ch#%d corr=%5.3f", iCh, gainCorr[iFile][iCh]);
      //gainCorr[iFile][iCh]=1.0;
    }
    printf("\n");
  }
}
//exit(-1);

dummy->SetBins(int(2.0*inputTrange), -50., inputTrange);
dummy->GetYaxis()->SetTitle("Input Current, mA");
inputCanvas -> Divide(2, 3, 0.005, 0.005);
dummyLine->SetLineWidth(2);

for (int iAtt=0; iAtt<nAttn; iAtt++) {
  iOff=nSizeLoadIn*iAtt;
  sprintf(title, "Attenuator 1:%.0f", Attenuator[iAtt]);
  dd=0.05*(inputMax[iAtt]-inputMin[iAtt]);
  dummy->SetTitle(title);
  dummy->GetYaxis()->SetRangeUser(inputMin[iAtt]-dd, inputMax[iAtt]+dd);
  dummy->GetYaxis()->SetTitleOffset(1.25);
  inputCanvas -> cd(iAtt+1);
  gPad->SetGridx(1);    
  gPad->SetGridy(1);
  gPad->SetRightMargin(0.03);
  gPad->SetLeftMargin(0.15);
  dummy->DrawCopy();
  for (int i=1; i<nSizeLoadIn; i++) {
    if (i*dTin[iAtt]-t0in>inputTrange) break;
    if (i*dTin[iAtt]-t0in<-50.) continue;
    dummyLine->DrawLine((i-1)*dTin[iAtt]-t0in, input[i+iOff-1],
                            i*dTin[iAtt]-t0in, input[i+iOff]);
  }
  
}
inputCanvas -> Update();
inputCanvas -> Show();

double ax=-1.0e10;
if (V_mA) ax=0.5;

//for (int jAtt=5; jAtt>-1; jAtt--) {
//for (int jAtt=0; jAtt<6; jAtt++) {
for (int jAtt=1; jAtt<6; jAtt+=4) { //quick debug
// show just the xtalks

  for (int i=0; i<nSizeLoad*nChan*nChan; i++) dataZ[i]=0.0;
  for (int i=0; i<nSizeLoad; i++) {
    dataSumX[i]=0.;
    dataSum[i]=0.;
  }

  double inMax=0.,  inMin=0.;
  double outMax=0., outMin=0.;
  int iStart=0;  
  for (int i=0; i<nSizeLoad; i++) {
    if (i<nSizeLoadIn) {
      input2[i]=input[i+jAtt*nSizeLoadIn];
      if (inMax<input2[i]) inMax=input2[i];
      if (inMin>input2[i]) inMin=input2[i];
    } else {
      input2[i]=0.;
    }
  }
  for (int i=0; i<nSizeLoadIn; i++) {
    if (input2[i]<0.5*(inMax+inMin)) {
      iStart=i;
      break;
    }
  }
  for (int i=0; i<nSizeLoad; i++) {
    int i1=i+iStart;
    if (i1>=nSizeLoad) i1-=nSizeLoad;
    input1[i]=input2[i1];
  }

  fftGetSpectrum(nSizeLoad,   bin001, freqCut1, fftMX, FFT1,   input1,  fftIin,  fftRin,  fftAin,  fftPin);
  fftFilter10MHz(nSizeLoad,   bin001, fftRin);
  fftFilter10MHz(nSizeLoad,   bin001, fftIin);
  fftFilter10MHz(nSizeLoad,   bin001, fftAin);
  fftFilter10MHz(nSizeLoad,   bin001, fftPin);

  for (double f0=2.; f0<200. && (smoothAmp || smoothPha); f0+=1.0) {
    if (smoothAmp) fftSmoothA(nSizeLoad, bin001, f0, 40., fftAin); 
    if (smoothPha) fftSmoothP(nSizeLoad, bin001, f0, 40., fftPin);
  } 

  double yMaxIN=0., yMinIN=0;
  for (int i=0; i<nSizeLoad/2 && i*bin001<freqCut1; i++) {
    if (yMaxIN<fftAin[i]) yMaxIN=fftAin[i]; 
    if (yMaxIN<fftRin[i]) yMaxIN=fftRin[i]; 
    if (yMaxIN<fftIin[i]) yMaxIN=fftIin[i]; 
    if (yMinIN>fftAin[i]) yMinIN=fftAin[i]; 
    if (yMinIN>fftRin[i]) yMinIN=fftRin[i]; 
    if (yMinIN>fftIin[i]) yMinIN=fftIin[i]; 
  }

/*
  sprintf(title, "Input spectra for attenuator 1:%.0f", Attenuator[jAtt]);
  spectraCanvasInput->Clear();
  spectraCanvasInput->Divide(1, 2);
  spectraCanvasInput->SetName(title);

  spectraCanvasInput->cd(1);
  gPad->SetLogx(1);
  gPad->SetGridx(1);
  gPad->SetGridy(1);

  dd = yMaxIN-yMinIN;
  dummyF->GetYaxis()->SetRangeUser(yMinIN-0.05*dd, yMaxIN+0.05*dd);
  sprintf(title, "Amplitude spectrum for attenuator 1:%.0f", Attenuator[jAtt]);
  dummyF->SetTitle(title);
  dummyF->DrawCopy();
  dummyLine->SetLineColor(kBlack);
  for (int i=1; i<nSizeLoad/2 && i*bin001<freqCut1; i++) {
    if ((i-1)*bin001<freqCut0) continue;
    dummyLine->DrawLine((i-1)*bin001, fftAin[i-1], i*bin001, fftAin[i]);
  }
  dummyLine->SetLineColor(kRed);
  for (int i=1; i<nSizeLoad/2 && i*bin001<freqCut1; i++) {
    if ((i-1)*bin001<freqCut0) continue;
    dummyLine->DrawLine((i-1)*bin001, fftRin[i-1], i*bin001, fftRin[i]);
  }
  dummyLine->SetLineColor(kBlue);
  for (int i=1; i<nSizeLoad/2 && i*bin001<freqCut1; i++) {
    if ((i-1)*bin001<freqCut0) continue;
    dummyLine->DrawLine((i-1)*bin001, fftIin[i-1], i*bin001, fftIin[i]);
  }

  spectraCanvasInput->cd(2);
  gPad->SetLogx(1);
  gPad->SetGridx(1);
  gPad->SetGridy(1);
  dummyF->GetYaxis()->SetRangeUser(-3.20, 3.20);
  sprintf(title, "Phase for attenuator 1:%.0f", Attenuator[jAtt]);
  dummyF->SetTitle(title);
  dummyF->DrawCopy();
  dummyLine->SetLineColor(kBlack);
  for (int i=1; i<nSizeLoad/2 && i*bin001<freqCut1; i++) {
    if ((i-1)*bin001<freqCut0) continue;
    dummyLine->DrawLine((i-1)*bin001, fftPin[i-1], i*bin001, fftPin[i]);
  }
  
  spectraCanvasInput->Update();
  sprintf(fileName, "InputSpectrum_Att%.0f_0912.ps", Attenuator[jAtt]);
  spectraCanvasInput->Print(fileName);
*/

//========================================================
  dummyLine->SetLineStyle(1);


  aMinG=-ax;
  aMaxG=ax;
  for (int iCh=0; iCh<4; iCh++) {
    aMin[iCh]=-ax;
    aMax[iCh]=ax;
  }
  cMinG=-ax;
  cMaxG=ax;
  for (int iCh=0; iCh<4; iCh++) {
    cMin[iCh]=-ax;
    cMax[iCh]=ax;
  }


  displayCanvas->Clear();
  sprintf(title, "X-talk 1:%.0f", Attenuator[jAtt]);
  displayCanvas->SetTitle(title);
  displayCanvas->Divide(2, 2, 0.005, 0.005);

  displayCanvasZoom->Clear();
  sprintf(title, "X-talk 1:%.0f, Zoomed", Attenuator[jAtt]);
  displayCanvasZoom->SetTitle(title);
  displayCanvasZoom->Divide(2, 2, 0.005, 0.005);

  for (int i=0; i<nSizeLoad*4; i++)           dataX[i]=0.0;
  for (int i=0; i<nSizeLoad; i++)             dataSumX[i]=0.0;
  

  for (int iCh=0; iCh<4; iCh++) {
    iOff=nSizeLoad*iCh;
    int jCh=0;
    for (int iFile=47; iFile<1000; iFile++) {

      if (iFilePulsed[iFile]==-100)           continue; // run in the logbook (.map file);
      if (iFilePulsed[iFile]==-1)             continue; // one channel is pulsed;
      if (iFilePulsed[iFile]==iCh+1)          continue; // it is NOT channel we are looking at now
      if (fileAtt[iFile] != Attenuator[jAtt]) continue; // attenuator is correct
      if (iFileCabled[iFile]!=iCh+1)          continue; // look at cabled channel only!
      int kCh=iFilePulsed[iFile]-1;

      iChScope=-1;
      for (int i=0; i<4; i++) {
        if (iMapScope[iFile][i]==iCh+1) {
          iChScope=i;
          break; 
        }
      }
      if (iChScope==-1) {
        printf("channel#%d not found in Scope Map for run=%d\n", iCh+1, iFile);
        exit(-1);
      } 


      sprintf(fileName, "2013.09.01/tek%04dCH%d.isf", iFile, iChScope+1);
/*
      if (loadFile(fileName, iOff, data, time, true)) {
        printf("X-talk chan#%d loaded file %s\n", iChScope+1, fileName);
      } else {
        printf("[Main] file %s got problems\n", fileName);
        exit(-1);
      }
*/
      if (!loadFile(fileName, iOff, data, time, true)) {
        printf("[Main] file %s got problems\n", fileName);
        exit(-1);
      }
      double w=0.0;
      ped0[iCh]=0.0;
      for (int i=0; i<nSizeLoad && time[i]<tDisp0; i++) {
        w+=1.0;
        ped0[iCh]+=data[i+iOff];
      }
      ped0[iCh]/=w;

      iOff1=nSizeLoad*(iCh+nChan*kCh);
      printf("setting X-talk for chan#%d from chan#%d(indx=%d) iOff1=%d\n", iCh, kCh, jCh, iCh+nChan*jCh); 
      for (int j=0; j<nSizeLoad; j++) {
        data[iOff+j]-=ped0[iCh];
        if (V_mA) data[iOff+j]/=inputCurrent[jAtt];
        dataX[iOff+j]+=data[iOff+j];
        dataSumX[j]  +=data[iOff+j]; 
        if (aMax[iCh]<data[iOff+j]) aMax[iCh]=data[iOff+j];
        if (aMin[iCh]>data[iOff+j]) aMin[iCh]=data[iOff+j];
        if (aMaxG<data[iOff+j])     aMaxG=data[iOff+j];
        if (aMinG>data[iOff+j])     aMinG=data[iOff+j];
        dataZ[iOff1+j]=data[iOff+j];
      }  
      jCh++;
    }
  }

  printf(" X-talk range = (%.3f, %.3f)  \n",
         aMinG, aMaxG);


  for (int iCh=0; iCh<4; iCh++) {
    iOff=nSizeLoad*iCh;
    dummy->SetBins(1000, tDisp0+cableDelay, tDisp1+cableDelay);
//------------------------
    aMax[iCh]=aMaxG; // same scale for all channels
    aMin[iCh]=aMinG;
//------------------------
    dd=0.06*(aMax[iCh]-aMin[iCh]);
    dummy->GetYaxis()->SetRangeUser(aMin[iCh]-dd, aMax[iCh]+dd);
    dummy->GetYaxis()->SetTitle("High Gain Output, V");
    if (V_mA) dummy->GetYaxis()->SetTitle("High Gain Response, V/mA");
    dummy->GetYaxis()->SetTitleOffset(1.55);
    displayCanvas->cd(iCh+1);
    gPad->SetGridx(1);    
    gPad->SetGridy(1);
    gPad->SetRightMargin(0.03);
    gPad->SetLeftMargin(0.15);
    sprintf(title, "Channel#%d X-talk", iCh+1);
    dummy->SetTitle(title);
    dummy->DrawCopy();

    dummy->SetBins(1050, tDisp0z+cableDelay, tDisp1z+cableDelay);
//------------------------
    aMax[iCh]=aMaxG; // same scale for all channels
    aMin[iCh]=aMinG;
//------------------------
    dd=0.06*(aMax[iCh]-aMin[iCh]);
    dummy->GetYaxis()->SetRangeUser(aMin[iCh]-dd, aMax[iCh]+dd);
    dummy->GetYaxis()->SetTitle("High Gain Output, V");
    if (V_mA) dummy->GetYaxis()->SetTitle("High Gain Response, V/mA");
    dummy->GetYaxis()->SetTitleOffset(1.55);
    displayCanvasZoom->cd(iCh+1);
    gPad->SetGridx(1);    
    gPad->SetGridy(1);
    gPad->SetRightMargin(0.03);
    gPad->SetLeftMargin(0.15);
    sprintf(title, "Channel#%d X-talk", iCh+1);
    dummy->SetTitle(title);
    dummy->DrawCopy();
  }


  int iY=1;
  dummyLatex->SetTextSize(0.05);
  dummyLatex->SetTextAlign(12);

  for (int iCh=0; iCh<4; iCh++) {
    iOff=nSizeLoad*iCh;
    for (int iFile=47; iFile<1000; iFile++) {

      if (iFilePulsed[iFile]==-100)           continue; // run in the logbook (.map file);
      if (iFilePulsed[iFile]==-1)             continue; // one channel is pulsed;
      if (iFilePulsed[iFile]==iCh+1)          continue; // it is NOT channel we are looking at now
      if (fileAtt[iFile] != Attenuator[jAtt]) continue; // attenuator is correct
      if (iFileCabled[iFile]!=iCh+1)          continue; // look at cabled channel only!


      iChScope=-1;
      for (int i=0; i<4; i++) {
        if (iMapScope[iFile][i]==iCh+1) {
          iChScope=i;
          break; 
        }
      }
      if (iChScope==-1) {
        printf("channel#%d not found in Scope Map for run=%d\n", iCh+1, iFile);
        exit(-1);
      } 

      sprintf(fileName, "2013.09.01/tek%04dCH%d.isf", iFile, iChScope+1);
      if (loadFile(fileName, iOff, data, time, true)) {
        printf(" loading file %s for Xtalk\n", fileName);
        yBin[iCh]=getYbin(fileName);
      } else {
        printf("[Main] file %s got problems\n", fileName);
        exit(-1);
      }
      double w=0.0;
      ped0[iCh]=0.0;
      for (int i=0; i<nSizeLoad && time[i]<tDisp0; i++) {
        w+=1.0;
        ped0[iCh]+=data[i+iOff];
      }
      ped0[iCh]/=w;
      for (int j=0; j<nSizeLoad; j++) {
        data[iOff+j]-=ped0[iCh];
        //data[iOff+j]*=gainCorr[iFile][iCh]; // no X-talk tuning!
        if (V_mA) data[iOff+j]/=inputCurrent[jAtt];
      }

      sprintf(title, "pulsed Chan#%d", iFilePulsed[iFile]);

      iY=iFilePulsed[iFile];
      displayCanvas->cd(iCh+1);
      dummyLatex->SetTextColor(iFilePulsed[iFile]);
      dummyLine ->SetLineColor(iFilePulsed[iFile]);
      dummyLine ->SetLineWidth(2);
      for (int i=1; i<nSizeLoad && time[i]<tDisp1+cableDelay; i++) {
        if (time[i-1]<tDisp0+cableDelay) continue;
        dummyLine->DrawLine(time[i-1], data[i+iOff-1], time[i], data[i+iOff]);
      }
      dummyLatex->DrawLatex(80.+cableDelay, 0.15*iY*aMin[iCh], title);

      displayCanvasZoom->cd(iCh+1);

      dummyLatex->SetTextColor(iFilePulsed[iFile]);
      dummyLine ->SetLineColor(iFilePulsed[iFile]);
      dummyLine ->SetLineWidth(2);
      for (int i=1; i<nSizeLoad && time[i]<tDisp1z+cableDelay; i++) {
        if (time[i-1]<tDisp0z+cableDelay) continue;
        dummyLine->DrawLine(time[i-1], data[i+iOff-1], time[i], data[i+iOff]);
      }
      dummyLatex->DrawLatex(50.+cableDelay, 0.15*iY*aMin[iCh], title);

      iStart=-1;
      for (int i=0; i<nSizeLoad; i++) {
        data2[i]=data[i+iOff]*Attenuator[jAtt]; // to keep normalization
        if (time[i]<214.) iStart=i;
      }
      for (int i=0; i<nSizeLoad; i++) {
        int i1=i+iStart;
        if (i1>=nSizeLoad) i1-=nSizeLoad;
        data1[i]=data2[i1];
      }

      fftGetSpectrum(nSizeLoad,   bin001, freqCut1, fftMX, FFT1,   data1,   fftIout, fftRout, fftAout, fftPout);
      fftFilter10MHz(nSizeLoad,   bin001, fftRout);
      fftFilter10MHz(nSizeLoad,   bin001, fftIout);
      fftFilter10MHz(nSizeLoad,   bin001, fftAout);
      fftFilter10MHz(nSizeLoad,   bin001, fftPout);
      for (double f0=2.; f0<200. && (smoothAmp || smoothPha); f0+=1.0) {
        if (smoothAmp) fftSmoothA(nSizeLoad, bin001, f0, 40., fftAout); 
        if (smoothPha) fftSmoothP(nSizeLoad, bin001, f0, 40., fftPout);
      } 
      fftGetConversion(nSizeLoad, bin001, freqCut1, fftMX, fftAin, fftAout, fftPin,  fftPout, slope,   shift);

      double yMax=0., yMin=0;
/*      
      spectraCanvasXtalk->Clear();
      spectraCanvasXtalk->Divide(1, 2);

      spectraCanvasXtalk->cd(1);
      gPad->SetLogx(1);
      yMax=0.;
      yMin=0.;
      for (int i=0; i<nSizeLoad/2 && i*bin001<freqCut1; i++) {
        if (yMax<fftAout[i]) yMax=fftAout[i]; 
        if (yMax<fftRout[i]) yMax=fftRout[i]; 
        if (yMax<fftIout[i]) yMax=fftIout[i]; 
        if (yMin>fftAout[i]) yMin=fftAout[i]; 
        if (yMin>fftRout[i]) yMin=fftRout[i]; 
        if (yMin>fftIout[i]) yMin=fftIout[i]; 
      }
      dd = yMax-yMin;
      dummyF->GetYaxis()->SetRangeUser(yMin-0.05*dd, yMax+0.05*dd);
      sprintf(title, "Amplitude spectrum for attenuator 1:%.0f", Attenuator[jAtt]);
      dummyF->SetTitle(title);
      dummyF->DrawCopy();
      dummyLine->SetLineColor(kBlack);
      for (int i=1; i<nSizeLoad/2 && i*bin001<freqCut1; i++) {
        if ((i-1)*bin001<freqCut0) continue;
        dummyLine->DrawLine((i-1)*bin001, fftAout[i-1], i*bin001, fftAout[i]);
      }

      dummyLine->SetLineColor(kRed);
      for (int i=1; i<nSizeLoad/2 && i*bin001<freqCut1; i++) {
        if ((i-1)*bin001<freqCut0) continue;
        dummyLine->DrawLine((i-1)*bin001, fftRout[i-1], i*bin001, fftRout[i]);
      }
      dummyLine->SetLineColor(kBlue);
      for (int i=1; i<nSizeLoad/2 && i*bin001<freqCut1; i++) {
        if ((i-1)*bin001<freqCut0) continue;
        dummyLine->DrawLine((i-1)*bin001, fftIout[i-1], i*bin001, fftIout[i]);
      }

      spectraCanvasXtalk->cd(2);
      gPad->SetLogx(1);
      dummyF->GetYaxis()->SetRangeUser(-3.20, 3.20);
      sprintf(title, "X-talk Phase for attenuator 1:%.0f", Attenuator[jAtt]);
      dummyF->SetTitle(title);
      dummyF->DrawCopy();
      dummyLine->SetLineColor(kBlack);
      for (int i=1; i<nSizeLoad/2 && i*bin001<freqCut1; i++) {
        if ((i-1)*bin001<freqCut0) continue;
        dummyLine->DrawLine((i-1)*bin001, fftPout[i-1], i*bin001, fftPout[i]);
      }

     
      spectraCanvasXtalk->Update();

//==========================================================
      if (iqq==1) {
        printf("push for next spectra! ->");
        cin>>iqq;
        if (iqq==-1) exit(-1);
        if (iqq==0) break;
      }
//==========================================================
*/
    } // file loop
  }   // channel loop
  
  aMinG=-ax;
  aMaxG=ax;
  for (int iCh=0; iCh<4; iCh++) {
    aMin[iCh]=-ax;
    aMax[iCh]=ax;
  }
  cMinG=-ax;
  cMaxG=ax;
  for (int iCh=0; iCh<4; iCh++) {
    cMin[iCh]=-ax;
    cMax[iCh]=ax;
  }
//==========================================================
// show miniFEB outputs of pulsed channel - alone and in group
//===========================================================

  displayCanvasSignal->Clear();
  sprintf(title, "Signal 1:%.0f", Attenuator[jAtt]);
  displayCanvasSignal->SetTitle(title);
  displayCanvasSignal->Divide(2, 2, 0.005, 0.005);

  displayCanvasSignalZoom->Clear();
  sprintf(title, "Signal 1:%.0f, Zoomed", Attenuator[jAtt]);
  displayCanvasSignalZoom->SetTitle(title);
  displayCanvasSignalZoom->Divide(2, 2, 0.005, 0.005);


  for (int iFile=47; iFile<1000; iFile++) {
    if (fileAtt[iFile] != Attenuator[jAtt]) continue; // only right attenuator
    if (iFileCabled[iFile] == 0)  continue;           // something is cabled

    for (int iCh=0; iCh<4; iCh++) {
      if (iFilePulsed[iFile]!=iCh+1 && iFilePulsed[iFile]!=-1) continue; // all or this one alone
      if (iFileCabled[iFile]!=iCh+1) continue; // pulsed other channel than cabled, skip

      iChScope=-1;
      for (int i=0; i<4; i++) {
        if (iMapScope[iFile][i]==iCh+1) {
          iChScope=i;
          break; 
        }
      }
      if (iChScope==-1) {
        printf("channel#%d not found in Scope Map for run=%d\n", iCh+1, iFile);
        exit(-1);
      } 

      sprintf(fileName, "2013.09.01/tek%04dCH%d.isf", iFile, iChScope+1);
      iOff=nSizeLoad*iCh;
      if (loadFile(fileName, iOff, data, time, true)) {
        printf("chan#%d loaded file %s\n", iCh+1, fileName);
        yBin[iCh]=getYbin(fileName);
      } else {
        printf("[Main] file %s got problems\n", fileName);
        exit(-1);
      }
      double w=0.0;
      ped0[iCh]=0.0;
      for (int i=0; i<nSizeLoad && time[i]<tDisp0; i++) {
        w+=1.0;
        ped0[iCh]+=data[i+iOff];
      }
      ped0[iCh]/=w;
      for (int j=0; j<nSizeLoad; j++) {
        data[iOff+j]-=ped0[iCh];         
        data[iOff+j]*=gainCorr[iFile][iCh];
        if (V_mA) data[iOff+j]/=inputCurrent[jAtt];
      }      

      for (int j=0; j<nSizeLoad; j++) {
        if (aMax[iCh]<data[iOff+j]) aMax[iCh]=data[iOff+j];
        if (aMin[iCh]>data[iOff+j]) aMin[iCh]=data[iOff+j];
        if (aMaxG<data[iOff+j]) aMaxG=data[iOff+j];
        if (aMinG>data[iOff+j]) aMinG=data[iOff+j];
      } // sample loop
    } // channel loop
  } // file loop

  printf(" Signal range = (%.3f, %.3f)  \n",
         aMinG, aMaxG);

  for (int iCh=0; iCh<4; iCh++) {
    iOff=nSizeLoad*iCh;
    dummy->SetBins(1000, tDisp0+cableDelay, tDisp1+cableDelay);
//------------------------
    aMax[iCh]=aMaxG; // same scale for all channels
    aMin[iCh]=aMinG;
//------------------------
    dd=0.02*(aMax[iCh]-aMin[iCh]);
    dummy->GetYaxis()->SetRangeUser(aMin[iCh]-dd, aMax[iCh]+dd);
    dummy->GetYaxis()->SetTitle("High Gain Output, V");
    if (V_mA) dummy->GetYaxis()->SetTitle("High Gain Response, V/mA");
    dummy->GetYaxis()->SetTitleOffset(1.55);
    displayCanvasSignal->cd(iCh+1);
    gPad->SetGridx(1);    
    gPad->SetGridy(1);
    gPad->SetRightMargin(0.03);
    gPad->SetLeftMargin(0.15);
    sprintf(title, "Channel#%d", iCh+1);
    dummy->SetTitle(title);
    dummy->DrawCopy();
    if (jAtt>jAtt3) {
      dummyLine->SetLineColor(kGray);
      dummyLine->SetLineWidth(5);
      double f=inputCurrent[jAtt]/inputCurrent[jAtt3];
      if (V_mA) f=1.;
      for (int i=1; i<nSizeLoad && time[i]<tDisp1+cableDelay; i++) {
        if (time[i-1]<tDisp0+cableDelay) continue;
        dummyLine->DrawLine(time[i-1], data3[i+iOff-1]*f, time[i], data3[i+iOff]*f); 
      }
      sprintf(title, "Scaled 1:%.0f signal", Attenuator[jAtt3]);
      dummyLatex->SetTextColor(kGray);
      dummyLatex->DrawLatex(50.+cableDelay, 0.1*aMax[iCh], title);
    }
    dummyLine->SetLineColor(kBlue);
    dummyLine->SetLineWidth(2);
    for (int i=1; i<nSizeLoad && time[i]<tDisp1+cableDelay; i++) {
      if (time[i-1]<tDisp0+cableDelay) continue;
      dummyLine->DrawLine(time[i-1], dataX[i+iOff-1], time[i], dataX[i+iOff]); 
    }
    dummyLatex->SetTextColor(kBlue);
    dummyLatex->DrawLatex(500.+cableDelay, -0.3*aMax[iCh], "X-talk");

    dummy->SetBins(1050, tDisp0z+cableDelay, tDisp1z+cableDelay);
    dummy->GetYaxis()->SetRangeUser(aMin[iCh]-dd, aMax[iCh]+dd);
    dummy->GetYaxis()->SetTitle("High Gain Output, V");
    if (V_mA) dummy->GetYaxis()->SetTitle("High Gain Response, V/mA");
    dummy->GetYaxis()->SetTitleOffset(1.55);
    displayCanvasSignalZoom->cd(iCh+1);
    gPad->SetGridx(1);    
    gPad->SetGridy(1);
    gPad->SetRightMargin(0.03);
    gPad->SetLeftMargin(0.15);
    sprintf(title, "Channel#%d", iCh+1);
    dummy->SetTitle(title);
    dummy->DrawCopy();
    if (jAtt>jAtt3) {
      double f1=inputCurrent[jAtt]/inputCurrent[jAtt3];
      if (V_mA) f1=1.;
      dummyLine->SetLineColor(kGray);
      dummyLine->SetLineWidth(5);
      for (int i=1; i<nSizeLoad && time[i]<tDisp1z+cableDelay; i++) {
        if (time[i-1]<tDisp0z+cableDelay) continue;
        dummyLine->DrawLine(time[i-1], data3[i+iOff-1]*f1, time[i], data3[i+iOff]*f1); 
      }
      sprintf(title, "Scaled 1:%.0f signal", Attenuator[jAtt3]);
      dummyLatex->SetTextColor(kGray);
      dummyLatex->DrawLatex(20.+cableDelay, 0.1*aMax[iCh], title);
    }
    dummyLine->SetLineColor(kBlue);
    dummyLine->SetLineWidth(2);
    for (int i=1; i<nSizeLoad && time[i]<tDisp1z+cableDelay; i++) {
      if (time[i-1]<tDisp0z+cableDelay) continue;
      dummyLine->DrawLine(time[i-1], dataX[i+iOff-1], time[i], dataX[i+iOff]); 
    }
    dummyLatex->SetTextColor(kBlue);
    dummyLatex->DrawLatex(70.+cableDelay, -0.3*aMax[iCh], "X-talk");

  }

  iY=1;
  dummyLatex->SetTextSize(0.05);
  dummyLatex->SetTextAlign(11);

  for (int iFile=47; iFile<1000; iFile++) {
    if (fileAtt[iFile] != Attenuator[jAtt]) continue;
    if (iFileCabled[iFile] == 0)  continue;           // no cable here

    if (iFilePulsed[iFile]==-1) {
      dummyLatex->SetTextColor(kBlack);
      sprintf(title, "All pulsed");
    } else {
      dummyLatex->SetTextColor(kRed);
      sprintf(title, "Only ch#%d pulsed", iFilePulsed[iFile]);
    }

    for (int iCh=0; iCh<4; iCh++) {

      if (iFilePulsed[iFile]!=iCh+1 && iFilePulsed[iFile]!=-1) continue;
      if (iFileCabled[iFile]!=iCh+1) continue; // pulsed other channel than cabled, skip

      iChScope=-1;
      for (int i=0; i<4; i++) {
        if (iMapScope[iFile][i]==iCh+1) {
          iChScope=i;
          break; 
        }
      }
      if (iChScope==-1) {
        printf("channel#%d not found in Scope Map for run=%d\n", iCh+1, iFile);
        exit(-1);
      } 
      sprintf(fileName, "2013.09.01/tek%04dCH%d.isf", iFile, iChScope+1);
      iOff=nSizeLoad*iCh;
      if (!loadFile(fileName, iOff, data, time, true)) {
        printf("[Main] file %s got problems\n", fileName);
        exit(-1);
      }

      iOff=nSizeLoad*iCh;
      double w=0.0;
      ped0[iCh]=0.0;
      for (int i=0; i<nSizeLoad && time[i]<tDisp0; i++) {
        w+=1.0;
        ped0[iCh]+=data[i+iOff];
      }
      ped0[iCh]/=w;      
      for (int i=0; i<nSizeLoad; i++) {
        data[iOff+i]-=ped0[iCh];
        data[iOff+i]*=gainCorr[iFile][iCh];
        if (V_mA) data[iOff+i]/=inputCurrent[jAtt];
        if (jAtt==jAtt3 && iFilePulsed[iFile]!=-1) data3[iOff+i]=data[iOff+i]; // save 1:1 signals
        if (iFilePulsed[iFile]==-1) {
          dataSum[i]+=data[iOff+i]; // All shall be pulsed for right X-talk in the Sum signal
        } else {
          data0[iOff+i]=data[iOff+i];
        }
      }

      displayCanvasSignal->cd(iCh+1);
      if (iFilePulsed[iFile]==-1) {      
        dummyLine->SetLineColor(kBlack);
        dummyLine->SetLineWidth(3);
        dummyLatex->DrawLatex(50.+cableDelay,  0.7*aMax[iCh], title);
      } else {
        dummyLine->SetLineColor(kRed);
        dummyLine->SetLineWidth(3);
        dummyLatex->DrawLatex(50.+cableDelay,  0.4*aMax[iCh], title);
      }
      for (int i=1; i<nSizeLoad && time[i]<tDisp1+cableDelay; i++) {
        if (time[i-1]<tDisp0+cableDelay) continue;
        dummyLine->DrawLine(time[i-1], data[i+iOff-1], time[i], data[i+iOff]); 
      }

      displayCanvasSignalZoom->cd(iCh+1);
      if (iFilePulsed[iFile]==-1) {      
        dummyLatex->DrawLatex(20.+cableDelay,  0.7*aMax[iCh], title);
      } else {
        dummyLatex->DrawLatex(20.+cableDelay,  0.4*aMax[iCh], title);
      }
      for (int i=1; i<nSizeLoad && time[i]<tDisp1z+cableDelay; i++) {
        if (time[i-1]<tDisp0z+cableDelay) continue;
        dummyLine->DrawLine(time[i-1], data[i+iOff-1], time[i], data[i+iOff]); 
      }

      iStart=-1;
      for (int i=0; i<nSizeLoad; i++) {
        data2[i]=data[i+iOff]*Attenuator[jAtt]; // accounting for input attenuator
        if (time[i]<214.) iStart=i;
      }
      for (int i=0; i<nSizeLoad; i++) {
        int i1=i+iStart;
        if (i1>=nSizeLoad) i1-=nSizeLoad;
        data1[i]=data2[i1];
      }

      fftGetSpectrum(nSizeLoad,   bin001, freqCut1, fftMX, FFT1,   data1,   fftIout, fftRout, fftAout, fftPout);
      fftFilter10MHz(nSizeLoad,   bin001, fftRout);
      fftFilter10MHz(nSizeLoad,   bin001, fftIout);
      fftFilter10MHz(nSizeLoad,   bin001, fftAout);
      fftFilter10MHz(nSizeLoad,   bin001, fftPout);
      for (double f0=2.; f0<200. && (smoothAmp || smoothPha); f0+=1.0) {
        if (smoothAmp) fftSmoothA(nSizeLoad, bin001, f0, 40., fftAout); 
        if (smoothPha) fftSmoothP(nSizeLoad, bin001, f0, 40., fftPout);
      } 
      fftGetConversion(nSizeLoad, bin001, freqCut1, fftMX, fftAin, fftAout, fftPin,  fftPout, slope,   shift);

//      if (iFilePulsed[iFile]==-1) { // save if all four are pulsed !
//        saveTransferFunction(dirTF, iCh, Attenuator[jAtt], nSizeLoad, bin001, slope, shift);
//      }
      iStart=-1; // do the X-talk transfer function ///
      for (int i=0; i<nSizeLoad; i++) {
        data2[i]=dataX[i+iOff]*Attenuator[jAtt]; // accounting for input attenuator
        if (time[i]<214.) iStart=i;
      }
      for (int i=0; i<nSizeLoad; i++) {
        int i1=i+iStart;
        if (i1>=nSizeLoad) i1-=nSizeLoad;
        data1[i]=data2[i1];
      }
      fftGetSpectrum(nSizeLoad,   bin001, freqCut1, fftMX, FFT1,   data1,   fftIout, fftRout, fftAout, fftPout);
      fftFilter10MHz(nSizeLoad,   bin001, fftRout);
      fftFilter10MHz(nSizeLoad,   bin001, fftIout);
      fftFilter10MHz(nSizeLoad,   bin001, fftAout);
      fftFilter10MHz(nSizeLoad,   bin001, fftPout);
      for (double f0=2.; f0<200. && (smoothAmp || smoothPha); f0+=1.0) {
        if (smoothAmp) fftSmoothA(nSizeLoad, bin001, f0, 40., fftAout); 
        if (smoothPha) fftSmoothP(nSizeLoad, bin001, f0, 40., fftPout);
      } 
      fftGetConversion(nSizeLoad, bin001, freqCut1, fftMX, fftAin, fftAout, fftPin,  fftPout, slopeX,   shiftX);

      dummyLine->SetLineColor(iCh+1);
      if (iFilePulsed[iFile]==-1) {
        dummyLine->SetLineWidth(3);   
      } else {
        continue; // skip runs where channels are pulsed one at a time
        dummyLine->SetLineWidth(1);   
      }
      
      

      if (iqq==0) break;
    } // channel loop
//    iY++;
    if (iqq==0) break;
  } // file loop 
  
//----------> find TF for Sum Signal and display it ///
 
  iStart=-1;
  for (int i=0; i<nSizeLoad; i++) {
    data2[i]=dataSum[i]*Attenuator[jAtt]/4.;// take into account that four channels are summed;
//    data2[i]=dataSum[i]*Attenuator[jAtt];
    if (time[i]<214.) iStart=i;
  }
  for (int i=0; i<nSizeLoad; i++) { // move signal into proper time position //
    int i1=i+iStart;
    if (i1>=nSizeLoad) i1-=nSizeLoad;
    data1[i]=data2[i1];
  }


  fftGetSpectrum(nSizeLoad,   bin001, freqCut1, fftMX, FFT1,   data1,   fftIout, fftRout, fftAout, fftPout);
  fftFilter10MHz(nSizeLoad,   bin001, fftRout);
  fftFilter10MHz(nSizeLoad,   bin001, fftIout);
  fftFilter10MHz(nSizeLoad,   bin001, fftAout);
  fftFilter10MHz(nSizeLoad,   bin001, fftPout);
  for (double f0=2.; f0<200. && (smoothAmp || smoothPha); f0+=1.0) {
    if (smoothAmp) fftSmoothA(nSizeLoad, bin001, f0, 40., fftAout); 
    if (smoothPha) fftSmoothP(nSizeLoad, bin001, f0, 40., fftPout);
  } 

  fftGetConversion(nSizeLoad, bin001, freqCut1, fftMX, fftAin, fftAout, fftPin,  fftPout, slope,   shift);

//  saveTransferFunction(dirTF, 4, Attenuator[jAtt], nSizeLoad, bin001, slope, shift);



//--- save low attenuation TF for future use

  for (int i=0; i<nSizeLoad; i++) {
//    slope[i]/=4.; // take into account that four channels are summed;
    if (jAtt==jAtt3) {
      slope3[i]=slope[i];
      shift3[i]=shift[i];
    }
  }

// restore input sum signal for full frequency band and 40MHz cut 
  fftRestore(nSizeLoad, bin001, 40.,    FFT1, FFT_BACK1, dataSum, fftIin, fftRin, fftAin, fftPin, slope, shift, reco_40);
  fftRestore(nSizeLoad, bin001, 1.0e10, FFT1, FFT_BACK1, dataSum, fftIin, fftRin, fftAin, fftPin, slope, shift, reco);
  for (int i=0; i<nSizeLoad; i++) {
    reco[i]   *=Attenuator[jAtt];
    reco_40[i]*=Attenuator[jAtt];
  }
  
  if (jAtt>jAtt3) {
    fftRestore(nSizeLoad, bin001, 40.,    FFT1, FFT_BACK1, dataSum, fftIin, fftRin, fftAin, fftPin, slope3, shift3, reco3_40);
    fftRestore(nSizeLoad, bin001, 1.0e10, FFT1, FFT_BACK1, dataSum, fftIin, fftRin, fftAin, fftPin, slope3, shift3, reco3);
    for (int i=0; i<nSizeLoad; i++) {
      reco3[i]   *=Attenuator[jAtt];
      reco3_40[i]*=Attenuator[jAtt];
    }
  }

  double recoMaxFull=4.0*inputMax[jAtt], recoMinFull=4.0*inputMin[jAtt];
  double recoMax40  =4.0*inputMax[jAtt], recoMin40=  4.0*inputMin[jAtt];
  for (int i=0; i<nSizeLoadIn; i++) {
    if (recoMaxFull<reco3[i]) recoMaxFull=reco3[i];
    if (recoMaxFull<reco[i])  recoMaxFull=reco[i];
    if (recoMinFull>reco3[i]) recoMinFull=reco3[i];
    if (recoMinFull>reco[i])  recoMinFull=reco[i];

    if (recoMax40<reco3_40[i]) recoMax40=reco3_40[i];
    if (recoMax40<reco_40[i])  recoMax40=reco_40[i];
    if (recoMin40>reco3_40[i]) recoMin40=reco3_40[i];
    if (recoMin40>reco_40[i])  recoMin40=reco_40[i];
  }
  printf(" Full Band signal swing=%.3f...%.3f\n", recoMinFull, recoMaxFull);
  printf("     40MHz signal swing=%.3f...%.3f\n", recoMin40, recoMax40);

  sprintf(title, "Reconstruction (500MHz)");
  dummy->SetTitle(title);
  dd=0.03*(recoMaxFull-recoMinFull);
  dummy->GetYaxis()->SetRangeUser(recoMinFull-dd, recoMaxFull+dd);
  dummy->GetYaxis()->SetTitleOffset(1.25);
  dummy->GetYaxis()->SetTitle("Input Current (Sum of 4 channels), mA");
  dummy->SetBins(10000, tDisp0, tDisp1);


  reconstructionFullBand->Clear();
  reconstructionFullBand->Divide(1,2);

  reconstructionFullBand->cd(2);
  gPad->SetGridx(1);    
  gPad->SetGridy(1);
  gPad->SetRightMargin(0.03);
  gPad->SetLeftMargin(0.15);
  dummy->DrawCopy();
  dummyLine->SetLineWidth(8);
  dummyLine->SetLineColor(kGray);
  iOff=nSizeLoadIn*jAtt;
  for (int i=1; i<nSizeLoadIn; i++) {
    if (i*dTin[jAtt]-t0in>tDisp1) break;
    if (i*dTin[jAtt]-t0in<tDisp0) continue;
    dummyLine->DrawLine((i-1)*dTin[jAtt]-t0in, 4.0*input[i+iOff-1],
                            i*dTin[jAtt]-t0in, 4.0*input[i+iOff]);
  }
  sprintf(title, "Gray - Input current");
  dummyLatex->SetTextColor(kGray);
  dummyLatex->DrawLatex(0.5*(tDisp0+tDisp1), 0.45*recoMinFull, title);
  
  
  reconstructionFullBand->cd(1);
  gPad->SetGridx(1);    
  gPad->SetGridy(1);
  gPad->SetRightMargin(0.03);
  gPad->SetLeftMargin(0.15);

  dd=0.03*(aMaxG-aMinG);
  dummy->GetYaxis()->SetRangeUser(4.0*(aMinG-dd), 4.0*(aMaxG+dd));
  dummy->GetYaxis()->SetTitle("Output Signal (Sum of 4 channels), V");
  dummy->SetBins(10000, tDisp0+cableDelay, tDisp1+cableDelay);
  sprintf(title, "Attenuator 1:%.0f", Attenuator[jAtt]);
  dummy->SetTitle(title);
  dummy->DrawCopy();

  if (jAtt != jAtt3) {
    dummyLine->SetLineWidth(6);
    dummyLine->SetLineColor(kGray);
    double f=inputCurrent[jAtt]/inputCurrent[jAtt3];
    for (int i=1; i<nSizeLoad && time[i]<tDisp1+cableDelay; i++) {
      if (time[i-1]<tDisp0+cableDelay) continue;
      double s0=f*(data3[i-1]+data3[i-1+nSizeLoad]+data3[i-1+2*nSizeLoad]+data3[i-1+3*nSizeLoad]);
      double s1=f*(data3[i]  +data3[i+nSizeLoad]  +data3[i+2*nSizeLoad]  +data3[i+3*nSizeLoad]);
      dummyLine->DrawLine(time[i-1], s0, time[i], s1);
    }
    sprintf(title, "Gray - Shape 1:3");
    dummyLatex->SetTextColor(kGray);
    dummyLatex->DrawLatex(0.5*(tDisp0+tDisp1)+cableDelay, 1.6*aMinG, title);
  }

  dummyLine->SetLineWidth(3);
  dummyLine->SetLineColor(kBlue);
  for (int i=1; i<nSizeLoad && time[i]<tDisp1+cableDelay; i++) {
    if (time[i-1]<tDisp0+cableDelay) continue;
    dummyLine->DrawLine(time[i-1], dataSumX[i-1], time[i], dataSumX[i]);
  }
  sprintf(title, "Blue - Crosstalk");
  dummyLatex->SetTextColor(kBlue);
  dummyLatex->DrawLatex(0.5*(tDisp0+tDisp1)+cableDelay, 2.0*aMinG, title);

  dummyLine->SetLineWidth(3);
  dummyLine->SetLineColor(kBlack);
  for (int i=1; i<nSizeLoad && time[i]<tDisp1+cableDelay; i++) {
    if (time[i-1]<tDisp0+cableDelay) continue;
    dummyLine->DrawLine(time[i-1], dataSum[i-1], time[i], dataSum[i]);
  }
  sprintf(title, "Black - Shape 1:%.0f", Attenuator[jAtt]);
  dummyLatex->SetTextColor(kBlack);
  dummyLatex->DrawLatex(0.5*(tDisp0+tDisp1)+cableDelay, 2.4*aMinG, title);
//----------------------------------------------------------------------------
  reconstructionFullBand->cd(2);
  dummyLine->SetLineWidth(4);
  dummyLine->SetLineColor(kBlack);
  for (int i=1; i<nSizeLoad && time[i]-214.<tDisp1; i++) {
    if (time[i-1]-214.<tDisp0) continue;
    dummyLine->DrawLine(time[i-1]-214., reco[i-1], time[i]-214., reco[i]);
  }
  reconstructionFullBand->Update();
  sprintf(title, "Black - 1:%.0f T.F. used",  Attenuator[jAtt]);
  dummyLatex->SetTextColor(kBlack);
  dummyLatex->DrawLatex(0.5*(tDisp0+tDisp1), 0.55*recoMinFull, title);

  if (jAtt>jAtt3) {
    dummyLine->SetLineWidth(2);
    dummyLine->SetLineColor(kRed);
    for (int i=1; i<nSizeLoad && time[i]-214.<tDisp1; i++) {
      if (time[i-1]-214.<tDisp0) continue;
      dummyLine->DrawLine(time[i-1]-214., reco3[i-1], time[i]-214., reco3[i]);
    }
    sprintf(title, "Red - 1:%.0f T.F.",  Attenuator[jAtt3]);
    dummyLatex->SetTextColor(kRed);
    dummyLatex->DrawLatex(0.5*(tDisp0+tDisp1), 0.70*recoMinFull, title);
  }
//----------------------------------------------------------------------------------

  sprintf(title, "Reconstruction (40MHz)");
  dummy->SetTitle(title);
  dd=0.03*(recoMax40-recoMin40);
  dummy->GetYaxis()->SetRangeUser(recoMin40-dd, recoMax40+dd);
  dummy->GetYaxis()->SetTitleOffset(1.25);
  dummy->GetYaxis()->SetTitle("Input Current (Sum of 4 channels), mA");
  dummy->SetBins(10000, tDisp0, tDisp1);


  reconstruction40MHz->Clear();
  reconstruction40MHz->Divide(1,2);

  reconstruction40MHz->cd(2);
  gPad->SetGridx(1);    
  gPad->SetGridy(1);
  gPad->SetRightMargin(0.03);
  gPad->SetLeftMargin(0.15);
  dummy->DrawCopy();
  dummyLine->SetLineWidth(8);
  dummyLine->SetLineColor(kGray);
  iOff=nSizeLoadIn*jAtt;
  for (int i=1; i<nSizeLoadIn; i++) {
    if (i*dTin[jAtt]-t0in>tDisp1) break;
    if (i*dTin[jAtt]-t0in<tDisp0) continue;
    dummyLine->DrawLine((i-1)*dTin[jAtt]-t0in, 4.0*input[i+iOff-1],
                            i*dTin[jAtt]-t0in, 4.0*input[i+iOff]);
  }
  sprintf(title, "Gray - Input current");
  dummyLatex->SetTextColor(kGray);
  dummyLatex->DrawLatex(0.5*(tDisp0+tDisp1), 0.45*recoMin40, title);
  
  
  reconstruction40MHz->cd(1);
  gPad->SetGridx(1);    
  gPad->SetGridy(1);
  gPad->SetRightMargin(0.03);
  gPad->SetLeftMargin(0.15);

  dd=0.03*(aMaxG-aMinG);
  dummy->GetYaxis()->SetRangeUser(4.0*(aMinG-dd), 4.0*(aMaxG+dd));
  dummy->GetYaxis()->SetTitle("Output Signal (Sum of 4 channels), V");
  dummy->SetBins(10000, tDisp0+cableDelay, tDisp1+cableDelay);
  sprintf(title, "Attenuator 1:%.0f", Attenuator[jAtt]);
  dummy->SetTitle(title);
  dummy->DrawCopy();

  if (jAtt != jAtt3) {
    dummyLine->SetLineWidth(6);
    dummyLine->SetLineColor(kGray);
    double f=inputCurrent[jAtt]/inputCurrent[jAtt3];
    for (int i=1; i<nSizeLoad && time[i]<tDisp1+cableDelay; i++) {
      if (time[i-1]<tDisp0+cableDelay) continue;
      double s0=f*(data3[i-1]+data3[i-1+nSizeLoad]+data3[i-1+2*nSizeLoad]+data3[i-1+3*nSizeLoad]);
      double s1=f*(data3[i]  +data3[i+nSizeLoad]  +data3[i+2*nSizeLoad]  +data3[i+3*nSizeLoad]);
      dummyLine->DrawLine(time[i-1], s0, time[i], s1);
    }
    sprintf(title, "Gray - Shape 1:3");
    dummyLatex->SetTextColor(kGray);
    dummyLatex->DrawLatex(0.5*(tDisp0+tDisp1)+cableDelay, 1.6*aMinG, title);
  }

  dummyLine->SetLineWidth(3);
  dummyLine->SetLineColor(kBlue);
  for (int i=1; i<nSizeLoad && time[i]<tDisp1+cableDelay; i++) {
    if (time[i-1]<tDisp0+cableDelay) continue;
    dummyLine->DrawLine(time[i-1], dataSumX[i-1], time[i], dataSumX[i]);
  }
  sprintf(title, "Blue - Crosstalk");
  dummyLatex->SetTextColor(kBlue);
  dummyLatex->DrawLatex(0.5*(tDisp0+tDisp1)+cableDelay, 2.0*aMinG, title);

  dummyLine->SetLineWidth(3);
  dummyLine->SetLineColor(kBlack);
  for (int i=1; i<nSizeLoad && time[i]<tDisp1+cableDelay; i++) {
    if (time[i-1]<tDisp0+cableDelay) continue;
    dummyLine->DrawLine(time[i-1], dataSum[i-1], time[i], dataSum[i]);
  }
  sprintf(title, "Black - Shape 1:%.0f", Attenuator[jAtt]);
  dummyLatex->SetTextColor(kBlack);
  dummyLatex->DrawLatex(0.5*(tDisp0+tDisp1)+cableDelay, 2.4*aMinG, title);
//----------------------------------------------------------------------------
  reconstruction40MHz->cd(2);
  dummyLine->SetLineWidth(4);
  dummyLine->SetLineColor(kBlack);
  for (int i=1; i<nSizeLoad && time[i]-214.<tDisp1; i++) {
    if (time[i-1]-214.<tDisp0) continue;
    dummyLine->DrawLine(time[i-1]-214., reco_40[i-1], time[i]-214., reco_40[i]);
  }
  reconstruction40MHz->Update();
  sprintf(title, "Black - 1:%.0f T.F. used",  Attenuator[jAtt]);
  dummyLatex->SetTextColor(kBlack);
  dummyLatex->DrawLatex(0.5*(tDisp0+tDisp1), 0.55*recoMin40, title);

  if (jAtt>jAtt3) {
    dummyLine->SetLineWidth(2);
    dummyLine->SetLineColor(kRed);
    for (int i=1; i<nSizeLoad && time[i]-214.<tDisp1; i++) {
      if (time[i-1]-214.<tDisp0) continue;
      dummyLine->DrawLine(time[i-1]-214., reco3_40[i-1], time[i]-214., reco3_40[i]);
    }
    sprintf(title, "Red - 1:%.0f T.F.",  Attenuator[jAtt3]);
    dummyLatex->SetTextColor(kRed);
    dummyLatex->DrawLatex(0.5*(tDisp0+tDisp1), 0.70*recoMin40, title);
  }


//----------------------------------------------------------------------------------
  reconstructionFullBand  -> Update();
  reconstruction40MHz     -> Update();
  displayCanvasSignal     -> Update();
  displayCanvasSignalZoom -> Update();
  displayCanvas           -> Update();
  displayCanvasZoom       -> Update();
/* 
  inputCanvas->Print("InputPulsesCabled0912.ps");
  sprintf(fileName, "Attenuator_1_%.0fCabled0912.ps(", Attenuator[jAtt]);
  displayCanvas->Print(fileName); 
  sprintf(fileName, "Attenuator_1_%.0fCabled0912.ps",  Attenuator[jAtt]);
  displayCanvasZoom  ->Print(fileName); 
  displayCanvasSignal->Print(fileName); 
  displayCanvasSignalZoom->Print(fileName); 
  reconstructionFullBand->Print(fileName); 
  sprintf(fileName, "Attenuator_1_%.0fCabled0912.ps)", Attenuator[jAtt]);
  reconstruction40MHz->Print(fileName); 

*/
//==========================================================
//  if (iqq<3) {
//    printf("push! ->");
//    cin>>iqq;
//    if (iqq==-1) exit(-1);
//  }
//  if (iqq==0) break;
} // attenuator loop


int jAtt=5; // simulations are all for 1:300 //  

simulationCanvas -> Divide(2,2);
aMaxG=0.;
aMinG=0.;
for (int i=0; i<nSizeLoad; i++) {
  if(aMinG>dataSum[i]) aMinG=dataSum[i];
  if(aMaxG<dataSum[i]) aMaxG=dataSum[i];
};
dd=0.06*(aMaxG-aMinG);
dummy->SetBins(1000, tDisp0+cableDelay, tDisp1+cableDelay);
//dummy->GetYaxis()->SetRangeUser(.25*(aMinG-dd), .25*(aMaxG+dd));
dummy->GetYaxis()->SetRangeUser(-0.7, 0.2);
dummy->GetYaxis()->SetTitle("High Gain Output, V");

for (int iCh=0; iCh<4; iCh++) {
  simulationCanvas -> cd(iCh+1);
  gPad->SetGridx(1);    
  gPad->SetGridy(1);
  gPad->SetRightMargin(0.03);
  gPad->SetLeftMargin(0.15);
  iOff=iCh*nSizeLoad;
  int iOff0=(iCh+iCh*nChan)*nSizeLoad;
  int jCh=0;
  sprintf(title, "Channel#%d", iCh+1);
  dummy->SetTitle(title);
  dummy->DrawCopy();
/*
  dummyLine->SetLineWidth(6);
  dummyLine->SetLineColor(kGray);
  for (int i=1; i<nSizeLoad && time[i]<tDisp1+cableDelay; i++) {
    if (time[i-1]<tDisp0+cableDelay) continue;
    dummyLine->DrawLine(time[i-1], .25*dataSum[i-1], time[i], .25*dataSum[i]);
  }
*/
  for (int i=1; i<nSizeLoad; i++) {
    dataZ[i+iOff0]=data0[i+iOff];
  } 
  for (int kCh=0; kCh<4; kCh++) {
    dummyLine->SetLineWidth(3);
    dummyLine->SetLineColor(kCh+1);
    iOff1=nSizeLoad*(iCh+nChan*kCh);
    printf("plotting X-talk for chan#%d from chan#%d(indx=%d) iOff1=%d\n", iCh, kCh, jCh, iCh+nChan*jCh);
    for (int i=1; i<nSizeLoad && time[i]<tDisp1+cableDelay; i++) {
      if (time[i-1]<tDisp0+cableDelay) continue;
      dummyLine->DrawLine(time[i-1], dataZ[i+iOff1-1], time[i], dataZ[i+iOff1]);
    }
    jCh++;
  }
  
}

simulationCanvas -> Update();

simulationCanvas1 -> Divide(1,2);
simulationCanvas1 -> cd(1);
dummy->SetBins(1000, tDisp0+cableDelay, tDisp1+cableDelay);
dummy->GetYaxis()->SetRangeUser(aMinG-3*dd, aMaxG+dd);
dummy->GetYaxis()->SetTitle("High Gain Output, V");
dummy->SetTitle("Output signal variation");
dummy->DrawCopy();
dummyLine->SetLineWidth(6);
dummyLine->SetLineColor(kRed);
for (int i=1; i<nSizeLoad && time[i]<tDisp1+cableDelay; i++) {
  if (time[i-1]<tDisp0+cableDelay) continue;
  dummyLine->DrawLine(time[i-1], dataSum[i-1], time[i], dataSum[i]);
}

simulationCanvas1 -> cd(2);
double recoMax=inputMax[jAtt], recoMin=inputMin[jAtt];
dd=recoMax-recoMin;
//dummy->GetYaxis()->SetRangeUser(recoMin-10*dd, recoMax+dd);
dummy->GetYaxis()->SetRangeUser(-24., 4.);
dummy->GetYaxis()->SetTitle("Reconstructed current, mA");
dummy->SetBins(10000, tDisp0, tDisp1);
dummy->SetTitle("Reconstructed detector current variation");
dummy->DrawCopy();
dummyLine->SetLineWidth(6);
dummyLine->SetLineColor(kRed);
iOff=nSizeLoadIn*jAtt;

double inputQ=0.;
double iMax=0.;
double recoQmin=1.0e10;
double recoQmax=-1.0e10;
for (int i=1; i<nSizeLoadIn; i++) {
  if (i*dTin[jAtt]-t0in>tDisp1) break;
  if (i*dTin[jAtt]-t0in<tDisp0) continue;
  if (iMax>input[i+iOff]) iMax=input[i+iOff];
  dummyLine->DrawLine((i-1)*dTin[jAtt]-t0in, 4.0*input[i+iOff-1],
                          i*dTin[jAtt]-t0in, 4.0*input[i+iOff]);
  if ((i*dTin[jAtt]-t0in)>-10. && (i*dTin[jAtt]-t0in)<50.) {
    inputQ+=input[i+iOff];
  }
}
iMax*=-4.;
inputQ*=-4.;
printf("Input charge in 50 ns = %fpC\n", inputQ);
//printf("push->");
//cin >> iqq;
if (iqq==0) exit(-1);

double wChan[4];
for (int iEvt=0; iEvt<5000; iEvt++) {
  if (iEvt>0 && iEvt%100==0) printf("Event#%d\n", iEvt);
  double sChan=0.0;
  //gRandom->RndmArray(4, wChan);
  for (int i=0; i<4; i++) {
    //wChan[i]=gRandom->Rndm();
    wChan[i]=gRandom->Exp(-1.);
    //wChan[i]/=2.0;
    sChan+=wChan[i];
  }
  sChan/=4.;
  for (int i=0; i<4; i++) {
    wChan[i]/=sChan;
    randomSpectrum->Fill(.25*wChan[i]);
  }
  for (int i=1; i<nSizeLoad; i++) {
    dataSumS[i]=0.0; 
    for (int iCh=0; iCh<4; iCh++) {
      iOff=nSizeLoad*iCh;
      dataChnS[i+iOff]=0.;
    }   
  }

  for (int iCh=0; iCh<4; iCh++) {
    iOff=nSizeLoad*iCh;
    for (int kCh=0; kCh<4; kCh++) {
      iOff1=nSizeLoad*(iCh+nChan*kCh);
      for (int i=1; i<nSizeLoad; i++) {
        double a=wChan[kCh]*dataZ[i+iOff1];
        dataChnS[i+iOff]+=a;
        dataSumS[i]+=a;   
      }
    }
  }

  fftRestore(nSizeLoad, bin001, 1.0e10, FFT1, FFT_BACK1, dataSumS, 
            fftIin, fftRin, fftAin, fftPin, slope, shift, reco);
//  fftRestore(nSizeLoad, bin001, 40., FFT1, FFT_BACK1, dataSumS, 
//             fftIin, fftRin, fftAin, fftPin, slope, shift, reco);

  double recP=0.;
  double resP=0.;
  double recQ=0.;
  for (int i=0; i<nSizeLoad; i++) {
    reco[i]   *=Attenuator[jAtt];
    if (reco[i]<recP) recP=reco[i];
    if (dataSumS[i]<resP) resP=dataSumS[i];
    if ((time[i]-214.)>-10. && (time[i]-214.)<50.) {
      recQ+=reco[i];
    }
  }
  //recQ*=1.0e12;
  recQ=-recQ;
  recoPeak->Fill(-recP);
  recoQ->Fill(recQ);
  outputPeak->Fill(-resP);

  if (recoQmax<recQ) {
    recoQmax=recQ;
    for (int i=0; i<nSizeLoad; i++) {
      recoMAX[i]=reco[i];
      outMAX[i]=dataSumS[i];
    }
    for (int iCh=0; iCh<4; iCh++) weightMAX[iCh]=wChan[iCh]/4.;
  }
  if (recoQmin>recQ) {
    recoQmin=recQ;
    for (int i=0; i<nSizeLoad; i++) {
      recoMIN[i]=reco[i];
      outMIN[i]=dataSumS[i];
    }
    for (int iCh=0; iCh<4; iCh++) weightMIN[iCh]=wChan[iCh]/4.;
  }
  
  
  if (iEvt<100) {
    printf(" recoI=%fmA recoQ=%fpC wMAX=(%5.3f, %5.3f, %5.3f, %5.3f)  wMIN=(%5.3f, %5.3f, %5.3f, %5.3f)\n",
            -recP, -recQ, 
            weightMAX[0], weightMAX[1], weightMAX[2], weightMAX[3], 
            weightMIN[0], weightMIN[1], weightMIN[2], weightMIN[3]);
    simulationCanvas1 -> cd(1);
    dummyLine->SetLineWidth(1);
    dummyLine->SetLineColor(kBlack);
    for (int i=1; i<nSizeLoad && time[i]<tDisp1+cableDelay; i++) {
      if (time[i-1]<tDisp0+cableDelay) continue;
      dummyLine->DrawLine(time[i-1], dataSumS[i-1], time[i], dataSumS[i]);
    }

    simulationCanvas1 -> cd(2);
    dummyLine->SetLineWidth(1);
    dummyLine->SetLineColor(kBlack);
    for (int i=1; i<nSizeLoad && time[i]-214<tDisp1; i++) {
      if (time[i-1]-214.<tDisp0) continue;
      dummyLine->DrawLine(time[i-1]-214., reco[i-1], 
                          time[i]-214.,   reco[i]);
    }
    simulationCanvas1 -> Update();
  }
}
//simulationCanvas1 -> cd(2);
//randomSpectrum->DrawCopy();

simulationCanvas1 -> cd(1);
dummyLine->SetLineWidth(3);
dummyLine->SetLineColor(kBlue);
for (int i=1; i<nSizeLoad && time[i]<tDisp1+cableDelay; i++) {
  if (time[i-1]<tDisp0+cableDelay) continue;
  dummyLine->DrawLine(time[i-1], outMAX[i-1], time[i], outMAX[i]);
}
for (int i=1; i<nSizeLoad && time[i]<tDisp1+cableDelay; i++) {
  if (time[i-1]<tDisp0+cableDelay) continue;
  dummyLine->DrawLine(time[i-1], outMIN[i-1], time[i], outMIN[i]);
}

simulationCanvas1 -> cd(2);
for (int i=1; i<nSizeLoad && time[i]-214<tDisp1; i++) {
  if (time[i-1]-214.<tDisp0) continue;
  dummyLine->DrawLine(time[i-1]-214., recoMAX[i-1], 
                      time[i]-214.,   recoMAX[i]);
}
for (int i=1; i<nSizeLoad && time[i]-214<tDisp1; i++) {
  if (time[i-1]-214.<tDisp0) continue;
  dummyLine->DrawLine(time[i-1]-214., recoMIN[i-1], 
                      time[i]-214.,   recoMIN[i]);
}

printf("=====> proportions for  Largest signal : (%5.3f, %5.3f, %5.3f, %5.3f)\n",
                           weightMAX[0], weightMAX[1], weightMAX[2], weightMAX[3]);
printf("=====> proportions for Smallest signal : (%5.3f, %5.3f, %5.3f, %5.3f)\n",
                           weightMIN[0], weightMIN[1], weightMIN[2], weightMIN[3]);


simulationCanvas1 -> Update();


simulationCanvas2 -> Divide(1,3);
//simulationCanvas2 ->cd(1);
//randomSpectrum->DrawCopy();

simulationCanvas2 ->cd(1);
outputPeak->DrawCopy();

simulationCanvas2 ->cd(2);
recoPeak->DrawCopy();
dummyLine->SetLineWidth(4);
dummyLine->SetLineColor(kRed);
dummyLine->DrawLine(iMax, 0., iMax, recoPeak->GetMaximum());
recoPeak->DrawCopy("same");

simulationCanvas2 ->cd(3);
recoQ->DrawCopy();
dummyLine->SetLineWidth(4);
dummyLine->SetLineColor(kRed);
dummyLine->DrawLine(inputQ, 0., inputQ, recoQ->GetMaximum());
recoQ->SetLineWidth(2);
recoQ->DrawCopy("same");
simulationCanvas2 -> Update();
}

//===============================================================
double getYbin(char *fileName) {
std::ifstream infile;
//printf("loading file %s\n", fileName);
infile.open(fileName, ifstream::in);
if (!infile.good()) {
  printf("[getYbin] error, %s is bad\n", fileName);
  infile.clear();
  return -100.;
}
bool preamble=true;
char preambLine[400];
for (int i=0; i<400; i++) preambLine[i]=' ';
int ipreamb=0;
double XIN=0.0; // time increment
double XZE=0.0; // t0     // t[sec]=XZE+XIN*(sample-1);
double YMU=0.0; // Y scale
double YOF=0.0; // raw data offset //
double YZE=0.0; // Y zero // y[V]=YZE+YMU*(data-YOF)

while (infile.good() && preamble) {
  preambLine[ipreamb] = (char) infile.get();
  //printf("[getYbin] -> PreambleLine[%d] -> %s\n", ipreamb, preambLine);
  if (preambLine[ipreamb]==';') { // end of preamble line, report, interpret and clear
    if (preambLine[0]=='X' &&
        preambLine[1]=='I' &&
        preambLine[2]=='N') {
      sscanf(preambLine, "%*s%lf", &XIN);
//      printf("XIN=%e\n", XIN);
    } else if (preambLine[0]=='X' &&
               preambLine[1]=='Z' &&
               preambLine[2]=='E') {
      sscanf(preambLine, "%*s%lf", &XZE);
//      printf("XZE=%e\n", XZE);
    } else if (preambLine[0]=='Y' &&
               preambLine[1]=='Z' &&
               preambLine[2]=='E') {
      sscanf(preambLine, "%*s%lf", &YZE);
      //printf("YZE=%e\n", YZE);
    } else if (preambLine[0]=='Y' &&
               preambLine[1]=='M' &&
               preambLine[2]=='U') {
      sscanf(preambLine, "%*s%lf", &YMU);
//      printf("YMU=%e\n", YMU);
    } else if (preambLine[0]=='Y' &&
               preambLine[1]=='O' &&
               preambLine[2]=='F') {
      sscanf(preambLine, "%*s%lf", &YOF);
      //printf("YOF=%e\n", YOF);
    } // preamble sentence was processed

    for (int i=0; i<400; i++) preambLine[i]=' ';
    ipreamb=0;
  } else if (preambLine[1]=='C' &&
             preambLine[2]=='U' &&
             preambLine[3]=='R' &&
             preambLine[4]=='V') { // curve signature 
    preamble=false;
  } else {
    ipreamb++;
  }
}
infile.close();
infile.clear();
return YMU;
}
//=========================================================================
bool loadFile(char *fileName, int iOff, double *data, double *time, bool getTime) {

std::ifstream infile;
//printf("loading file %s\n", fileName);
infile.open(fileName, ifstream::in);
if (!infile.good()) {
  printf("[loadFile] error, %s is bad\n", fileName);
  infile.clear();
  return false;
}
if (!infile.is_open()) {
  printf("[loadFile] error, %s is not open\n", fileName);
  infile.clear();
  return false;
}
//short word;
short int word;
unsigned char Bite1,Bite2;
bool preamble=true;
char preambLine[500];
char inchar;

for (int i=0; i<500; i++) preambLine[i]=' ';

int wordCount=0;
int ipreamb=0;
int inchars=0;
int inbytes=0;
double XIN=0.0; // time increment
double XZE=0.0; // t0     // t[sec]=XZE+XIN*(sample-1);
double YMU=0.0; // Y scale
double YOF=0.0; // raw data offset //
double YZE=0.0; // Y zero // y[V]=YZE+YMU*(data-YOF)

while (infile.good()) {
  if (preamble) {
    preambLine[ipreamb] = (char) infile.get();
    //printf("[loadFile] -> PreambleLine[%d] -> %s\n", ipreamb, preambLine);
    if (preambLine[ipreamb]==';') { // end of preamble line, report, interpret and clear
      if (preambLine[0]=='X' &&
          preambLine[1]=='I' &&
          preambLine[2]=='N') {
        sscanf(preambLine, "%*s%lf", &XIN);
        //printf("XIN=%e\n", XIN);
      } else if (preambLine[0]=='X' &&
                 preambLine[1]=='Z' &&
                 preambLine[2]=='E') {
        sscanf(preambLine, "%*s%lf", &XZE);
        //printf("XZE=%e\n", XZE);
      } else if (preambLine[0]=='Y' &&
                 preambLine[1]=='Z' &&
                 preambLine[2]=='E') {
        sscanf(preambLine, "%*s%lf", &YZE);
        //printf("YZE=%e\n", YZE);
      } else if (preambLine[0]=='Y' &&
                 preambLine[1]=='M' &&
                 preambLine[2]=='U') {
        sscanf(preambLine, "%*s%lf", &YMU);
        //printf("YMU=%e\n", YMU);
      } else if (preambLine[0]=='Y' &&
                 preambLine[1]=='O' &&
                 preambLine[2]=='F') {
        sscanf(preambLine, "%*s%lf", &YOF);
        //printf("YOF=%e\n", YOF);
      } // preamble sentence was processed
      for (int i=0; i<400; i++) preambLine[i]=' ';
      ipreamb=0;
    } else if (preambLine[1]=='C' &&
               preambLine[2]=='U' &&
               preambLine[3]=='R' &&
               preambLine[4]=='V') { // curve signature 
      preamble=false;
      inchar = (char) infile.get();
//cout << "1 " << inchar << endl;
      inchar = (char) infile.get();
//cout << "2 " << inchar << endl;
      inchar = infile.get();
//cout << "3 " << inchar << endl;
      inchars=0;
      sscanf(&inchar, "%1d", &inchars);

      inbytes=0;
//cout << "5 " << inchars << endl;
//printf("number of characters in byte count = %d \n", inchars);
      int word0;
      for (int i=0; i<inchars; i++) {
        inchar = infile.get();
//cout << "6 " << inchar << endl;
        sscanf(&inchar, "%1d", &word0);
        inbytes = 10*inbytes + word0;
//cout << "7 " << word0 << " " << inbytes << endl;
      }
//      printf("%d bytes to be loaded ->\n", inbytes);
    } else {
      ipreamb++;
    }
  } else {
    word=0;
    //short word,word1,word2;
    Bite1=infile.get(); 
    Bite2=infile.get();
    //word1=Bite1; 
    //word2=Bite2;
    word=256*Bite1+Bite2;
    
    if (infile.good()) {
      data[iOff+wordCount]=YZE+YMU*(word-YOF);
      if (getTime) time[wordCount]=1.0e9*(XZE+XIN*(wordCount-1));
      wordCount++;
    }
  }      
}
//printf("[loadFile] %d binary words loaded from %s\n", wordCount, fileName);
infile.close();
infile.clear();
//printf("[loadFile] done loading file%s\n", fileName);
if (wordCount*2 != inbytes) {
  printf("[loadFile] wrong byte count! words called %d, bytes loaded %d\n", wordCount, inbytes);
  return false;
}
return true;
}

double fitIN(double *par, double t) {
  double a=par[0];
  if (t>par[3]) { 
    if (t<par[4]) {
      a+=par[1]*(par[4]-t)/(par[4]-par[3]);
    }
  } else if (t>par[2]) {
    a+=par[1]*(t-par[2])/(par[3]-par[2]);
  }
  return a;
}


double fitTail(double *par, double t) {
  double a=par[0]+par[1]/par[2]*exp(-t/par[2])*cos(pi2*(t-par[4])/par[3]);
  return a;
}

void fftRestore(int nSamp, double fBin, double fCut, TVirtualFFT *FFT, TVirtualFFT *FFTback, 
                double *output, double *aI,    double *aR,  double *aA,  double *aP, 
                double *slope,  double *shift, double *input) {
// nSamp - number of signal samples
// fBin  - frequency bin size
// fCut  - frequency cut  
// FFT, FFTback - pointers to FFT transforms to use
// output       - [nSamp] signal "digitised by ADC"
// aI, aR       - [nSamp] imaginary and real amplitudes
// aA, aP       - [nSamp] amplitude and phase spetra, derived fro, aI, aR using nSamp/2 values
// scale, shift - [nSamp/2] modificators for aA and aP, found from single pulse bench studies 
// intput       - [nSamp] reconctructed input signal
  for (int i=0; i<nSamp; i++) {
    aI[i]=0.;
    aR[i]=0.;
    aA[i]=0.;
    aP[i]=0.;
    input [i]=0.;
  }

  FFT->SetPoints(output);
  FFT->Transform();
  FFT->GetPointsComplex(aR, aI);
  for (int i=0; i<nSamp; i++) {
    aR[i]/=(double)nSamp;
    aI[i]/=(double)nSamp;
  }

  for (int i=0; i<nSamp/2+1; i++) {
    aA[i]=sqrt(aR[i]*aR[i]+aI[i]*aI[i]); 
    aP[i]=atan2(aI[i], aR[i]);

    aA[i]/=slope[i];
    if (i*fBin>fCut) aA[i]=0.0;
    aP[i]-=shift[i];
    aR[i]=aA[i]*cos(aP[i]);
    aI[i]=aA[i]*sin(aP[i]);
  }

    //fftRhg[0]=0.;
    //fftIhg[0]=0.;
  FFTback->SetPointsComplex(aR, aI);
  FFTback->Transform();
  FFTback->GetPoints(input); 
}


void MyExecuteEvent(Int_t event, Int_t px, Int_t py, TObject *sel)
{
  if(sel) {
    //
  }
  TCanvas *c = (TCanvas *) gTQSender;

  if ( (event == kButton1Double) ) {
    char cname[256];
    TPad *pad = c->Pick(px, py, 0);
    //printf("d-clicked on pad %s #%d\n", pad->GetName(), pad->GetNumber());
    if(pad == 0) {
      std::cout << "Can;t get pad at " << px << " " << py << std::endl;
      return;
    }
    if(pad == c) {
      std::cout << "You have selected parent object" << std::endl;
      return;
    }
    float xyratio =  float(pad->GetAbsWNDC()*pad->GetCanvas()->GetWindowWidth()) / float(pad->GetAbsHNDC()*pad->GetCanvas()->GetWindowHeight());
    //std::cout << " xyratio: " << xyratio << std::endl;
    TPad *pad_new = (TPad *) pad->Clone();
    pad_new->SetPad(0.0, 0.0, 1.0, 1.0);
    //pad_new -> SetEditable(true);

    sprintf(cname,"%s_%d",pad->GetTitle(), (int)time(0));
    int iY=600;
    int iX=(int)(600*xyratio);
    if (iX<600) {
      iY=iY*600/iX;
      iX=600;
    }
    TCanvas *c1 = new TCanvas(cname, cname, iX, iY);
    c1->cd();
    if (cname[0]=='S' && cname[1]=='p' && cname[2]=='e' && cname[3]=='c') {
      gStyle->SetOptStat(111110);
    } 
    pad_new->SetEditable(kTRUE);
    pad_new->Draw();    
    pad_new->Modified();
    pad_new->Update();
//    pad_new->SetEditable(kFALSE);
//     c1->RaiseWindow();
//     c1->Modified();
//     c1->Update();
    c1->SetWindowSize(iX+20, iY+40);
    c1->Update();
    gSystem->ProcessEvents();
  }
} 

void saveTransferFunction(char* dirTF, int iCh, double Att, 
    int nSizeLoad, double bin001, double *slope, double *shift) {

char fileName[400];
sprintf(fileName, "%schan%datt%.0f.dat", dirTF, iCh, Att);
printf("[saveTransferFunction] file %s\n", fileName);
outfile.open(fileName, ios::out);
outfile << nSizeLoad << " " << bin001 << " " << bin001 << endl;
for (int i=0; i<nSizeLoad/2+2; i++) {
   outfile << slope[i] << " " << shift[i] << endl;
}
outfile.close();
outfile.clear();
}


